import { SObjectItem, SImageItem, SGraphItem, STextItem } from "@saga-web/graph/lib";
import { ItemOrder } from "@saga-web/big/lib";
import { SColor, SPainter, SRect } from "@saga-web/draw/lib";
import { SMouseEvent } from "@saga-web/base/lib";
import { EquipmentData } from "../types/EquipmentData";
enum StatusType {
    /** 普通态    */
    Normal,
    /** 编辑态    */
    Edit
}
export class SEquipmentItem extends SObjectItem {
    // img对象
    img: SImageItem = new SImageItem(this);
    _textArr: STextItem[] = [];
    /** imgkey值 */
    _imgKey: string = "";
    /** 设备状态值,开启:关闭:故障... */
    _status: string = "Running";
    /**保存设备数据 */
    _epData: EquipmentData | any = {};
    /** 设备名称 */
    _name: string = '';
    /** 设备状态:编辑,正常 */
    equipmentStatus: StatusType = StatusType.Normal;

    get imgKey(): string {
        return this._imgKey;
    }
    set imgKey(v: string) {
        this._imgKey = v;
        // TODO: 
        // this.img.url = `/serve/topology-wanda/Picture/query/${v}`;
        this.img.url = `http://60.205.177.43/topolImages/${v}`;
        this.img.connect("imgLoadOver", this, () => {
            // 抛出设备中的图片加载完成事件
            this.$emit("equipImgLoadOver");
            this.update();
        });
    }
    get status(): string {
        return this._status;
    }
    set status(v: string) {
        this._status = v;
        this.updateStatus();
    }
    get textArr(): STextItem[] {
        return this._textArr
    }
    set textArr(v) {
        this._textArr = v
    }
    // 将EquipmentData保存起来,以便修改status时,可以找到StatusImage
    get epData(): EquipmentData {
        return this._epData;
    }
    set epData(v: EquipmentData) {
        this._epData = v;


    }
    // 修改设备名称
    get name(): string {
        return this._name
    }
    set name(v: string) {
        this._name = v
        if (this.textArr && this.textArr.length) {
            this.textArr[0].text = v
        }
    }
    _info: any[] = []
    // 修改信息点名称
    get info(): [] {
        return this._info
    }
    set info({ index, name }) {
        if (this.textArr && this.textArr.length > 1) {
            if (this.textArr[index + 1]){

                this.textArr[index + 1].text = name
            }else{
                this.textArr[index + 1] = 
            }
        }
    }

    /**
     *
     * @param parent 父类
     * @param epData  需要展示的节点信息
     */
    constructor(parent: SGraphItem | null, epData: EquipmentData) {
        super(parent);
        console.log(epData);
        epData && (this.epData = epData);
        this.moveable = true;
        this.id = epData.ID;
        this.zOrder = ItemOrder.textOrder;
        this.textArr = [];
        // 有信息点信息时,绘制信息点信息
        // 加载设备
        if (epData && epData?.Pos) {
            this.loadEquipment(epData);
        } else if (epData) {
            //绘制设备
            this.img.url = `/serve/topology-wanda/Picture/query/14d978b7edd346f088d6cfb53ada4070`;
            this.img.width = 24;
            this.img.height = 24;
            //添加设备
            let index = 0;
            for (let key in epData) {
                let textItem = new STextItem(this);
                textItem.moveable = true;
                textItem.backgroundColor = SColor.White;
                textItem.font.size = 14;
                textItem.moveTo(17, index * 20 - 10);
                // @ts-ignore
                textItem.text = `${key} : ${epData[key]}`;
                this.textArr.push(textItem);
                index++;
            }
        }
    }
    /**
     * 加载设备
     * @param epData 设备信息
     */
    loadEquipment(epData: EquipmentData): void {
        // 位置
        this.x = epData.Pos.X;
        this.y = epData.Pos.Y;
        let imageKey = epData.Properties?.StatusImage![0].ImageKey;
        // this.img.url = `http://localhost:8091/serve/topology-wanda/Picture/query/${url}`;
        this.imgKey = imageKey;
        // 图片宽高
        this.img.width = epData.Size.Width;
        this.img.height = epData.Size.Height;
        // 名称
        this.name = epData.Name
        let nameItem = new STextItem(this);
        // nameItem.moveable = true;
        nameItem.backgroundColor = SColor.Transparent;
        nameItem.font.size = 14;
        nameItem.text = epData.Name;
        // nameItem.moveTo(-epData.Size.Width / 2, epData.Size.Height / 2 + 7);
        nameItem.moveTo(10, epData.Size.Height + 7);
        this.textArr.push(nameItem);
        // 信息点
        let { InfoList } = epData;
        InfoList.map(({ Name, X, Y, Width, Height, FontSize, Background, Color, TextAlign }) => {
            let textItem = new STextItem(this);
            textItem.backgroundColor = new SColor(Background);
            textItem.width = Width;
            textItem.height = Height;
            textItem.text = Name;
            textItem.font.size = FontSize;
            textItem.color = new SColor(Color);
            textItem.pos.x = X;
            textItem.pos.y = Y;
            this.textArr.push(textItem);
        });
    }
    /**
     * 状态变化后,更新状态切换图片
     * 如果切换后的状态无ImageKey,使用Running状态的ImageKey
     */
    updateStatus() {
        let StatusImage = this.epData.Properties.StatusImage,
            currentSts = StatusImage.filter((v) => v.Status === this._status)[0],
            runingSts = StatusImage.filter((v) => v.Status === "Running")[0];
        this.imgKey = currentSts.ImageKey || runingSts.ImageKey;
    }
    /**
   * 鼠标双击事件
   *
   * @param	event         事件参数
   * @return	boolean
   */
    onDoubleClick(event: SMouseEvent): boolean {
        // 如果位show状态 双击改对象则需改为编辑状态
        if (StatusType.Normal == this.equipmentStatus) {
            this.equipmentStatus = StatusType.Edit;
            // this.grabItem(this); // TODO: 
        } else if (StatusType.Edit == this.equipmentStatus) {
            this.equipmentStatus = StatusType.Normal;
            this.releaseItem();
        }
        this.update()
        return true;
    } // Function onDoubleClick()


    /**
     * 宽高发生变化
     *
     * @return  SRect   所有子对象的并集
     * */
    boundingRect(): SRect {
        //  return new SRect(-this.width / 2, -this.height / 2, this.width, this.height);
        let rect = this.img.boundingRect().adjusted(this.img.pos.x, this.img.pos.y, 0, 0);
        if (this.textArr.length) {
            for (let text of this.textArr) {
                rect = rect.unioned(text.boundingRect().adjusted(text.pos.x, text.pos.y, 0, 0));
            }
        }
        return rect.adjusted(-5, -5, 10, 10);
    } // Function boundingRect()
    toData(): EquipmentData {
        //  更新相关数据
        let data = this.epData;
        data.Pos.X = this.x;
        data.Pos.Y = this.y;
        data.Size.Width = this.width;
        data.Size.Height = this.height;
        // @ts-ignore
        // infoList中坐标,名称处理
        data.InfoList.map((item, index, arr) => {
            arr[index].X = this.textArr[index + 1].pos.x;
            arr[index].Y = this.textArr[index + 1].pos.y;
            arr[index].Name = this.textArr[index + 1].text;
        });
        return data;
    }

    /**
     * Item绘制操作
     *
     * @param   painter painter对象
     */
    onDraw(painter: SPainter): void {
        painter.pen.lineWidth = painter.toPx(1);
        painter.pen.color = SColor.Black;
        painter.brush.color = SColor.Transparent;
        if (this.equipmentStatus === StatusType.Edit) {
            painter.drawRect(this.boundingRect());
            this.textArr.map(item => item.moveable = true)
        } else {
            this.textArr.map(item => item.moveable = false)

        }
    } // Function onDraw()
}
